import os
import glob
import numpy as np
from random import shuffle
import multiprocessing as mp
import itertools

from astrodash.helpers import temp_list, div0
from astrodash.sn_processing import PreProcessing
from astrodash.combine_sn_and_host import training_template_data
from astrodash.preprocessing import ProcessingTools
from astrodash.array_tools import zero_non_overlap_part, normalise_spectrum

try:
    from imblearn import over_sampling
    IMBLEARN_EXISTS = True
except ImportError:
    print("Need to install imblearn `pip install imblearn` to use smote.")
    IMBLEARN_EXISTS = False


class AgeBinning(object):
    def __init__(self, minAge, maxAge, ageBinSize):
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize

    def age_bin(self, age):
        ageBin = int(round(age / self.ageBinSize)) - int(round(self.minAge / self.ageBinSize))

        return ageBin

    def age_labels(self):
        ageLabels = []

        ageBinPrev = 0
        ageLabelMin = self.minAge
        for age in np.arange(self.minAge, self.maxAge, 0.5):
            ageBin = self.age_bin(age)

            if ageBin != ageBinPrev:
                ageLabelMax = int(round(age))
                ageLabels.append(str(int(ageLabelMin)) + " to " + str(ageLabelMax))
                ageLabelMin = ageLabelMax

            ageBinPrev = ageBin

        ageLabels.append(str(int(ageLabelMin)) + " to " + str(int(self.maxAge)))

        return ageLabels


class CreateLabels(object):

    def __init__(self, nTypes, minAge, maxAge, ageBinSize, typeList, hostList, nHostTypes):
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.ageBinning = AgeBinning(self.minAge, self.maxAge, self.ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(self.maxAge - 0.1) + 1
        self.nLabels = self.nTypes * self.numOfAgeBins
        self.ageLabels = self.ageBinning.age_labels()
        self.hostList = hostList
        self.nHostTypes = nHostTypes

    def label_array(self, ttype, age, host=None):
        ageBin = self.ageBinning.age_bin(age)

        try:
            typeIndex = self.typeList.index(ttype)
        except ValueError as err:
            raise Exception("INVALID TYPE: {0}".format(err))

        if host is None:
            labelArray = np.zeros((self.nTypes, self.numOfAgeBins))
            labelArray[typeIndex][ageBin] = 1
            labelArray = labelArray.flatten()
            typeName = ttype + ": " + self.ageLabels[ageBin]
        else:
            hostIndex = self.hostList.index(host)
            labelArray = np.zeros((self.nHostTypes, self.nTypes, self.numOfAgeBins))
            labelArray[hostIndex][typeIndex][ageBin] = 1
            labelArray = labelArray.flatten()
            typeName = "{}: {}: {}".format(host, ttype, self.ageLabels[ageBin])

        labelIndex = np.argmax(labelArray)

        return labelIndex, typeName

    def type_names_list(self):
        typeNamesList = []
        if self.hostList is None:
            for tType in self.typeList:
                for ageLabel in self.ageBinning.age_labels():
                    typeNamesList.append("{}: {}".format(tType, ageLabel))
        else:
            for host in self.hostList:
                for tType in self.typeList:
                    for ageLabel in self.ageBinning.age_labels():
                        typeNamesList.append("{}: {}: {}".format(host, tType, ageLabel))

        return np.array(typeNamesList)


class ReadSpectra(object):

    def __init__(self, w0, w1, nw, snFilename, galFilename=None):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.snFilename = snFilename
        if galFilename is None:
            self.data = PreProcessing(snFilename, w0, w1, nw)
        else:
            self.galFilename = galFilename

    def sn_plus_gal_template(self, snAgeIdx, snCoeff, galCoeff, z):
        wave, flux, minIndex, maxIndex, nCols, ages, tType = training_template_data(snAgeIdx, snCoeff, galCoeff, z,
                                                                                    self.snFilename, self.galFilename,
                                                                                    self.w0, self.w1, self.nw)

        return wave, flux, nCols, ages, tType, minIndex, maxIndex

    def input_spectrum(self, z, smooth, minWave, maxWave):
        wave, flux, minIndex, maxIndex, z = self.data.two_column_data(z, smooth, minWave, maxWave)

        return wave, flux, int(minIndex), int(maxIndex), z


class ArrayTools(object):

    def __init__(self, nLabels, nw):
        self.nLabels = nLabels
        self.nw = nw

    def shuffle_arrays(self, memmapName='', **kwargs):
        """ Must take images and labels as arguments with the keyword specified.
        Can optionally take filenames and typeNames as arguments """
        arraySize = len(kwargs['labels'])
        kwargShuf = {}
        self.randnum = np.random.randint(10000)
        for key in kwargs:
            if key == 'images':
                arrayShuf = np.memmap('shuffled_{}_{}_{}.dat'.format(key, memmapName, self.randnum), dtype=np.float16,
                                      mode='w+', shape=(arraySize, int(self.nw)))
            elif key == 'labels':
                arrayShuf = np.memmap('shuffled_{}_{}_{}.dat'.format(key, memmapName, self.randnum), dtype=np.uint16,
                                      mode='w+', shape=arraySize)
            else:
                arrayShuf = np.memmap('shuffled_{}_{}_{}.dat'.format(key, memmapName, self.randnum), dtype=object,
                                      mode='w+', shape=arraySize)
            kwargShuf[key] = arrayShuf

        print("Shuffling...")
        # Randomise order
        p = np.random.permutation(len(kwargs['labels']))
        for key in kwargs:
            assert len(kwargs[key]) == arraySize
            print(key, "shuffling...")
            print(len(p))
            kwargShuf[key] = kwargs[key][p]

        return kwargShuf

    def count_labels(self, labels):
        counts = np.zeros(self.nLabels)

        for i in range(len(labels)):
            counts[labels[i]] += 1

        return counts

    def augment_data(self, flux, stdDevMean=0.05, stdDevStdDev=0.05):
        minIndex, maxIndex = ProcessingTools().min_max_index(flux, outerVal=0.5)
        noise = np.zeros(self.nw)
        stdDev = abs(np.random.normal(stdDevMean, stdDevStdDev))  # randomised standard deviation
        noise[minIndex:maxIndex] = np.random.normal(0, stdDev, maxIndex - minIndex)
        # # Add white noise to regions outside minIndex to maxIndex
        # noise[0:minIndex] = np.random.uniform(0.0, 1.0, minIndex)
        # noise[maxIndex:] = np.random.uniform(0.0, 1.0, self.nw-maxIndex)

        augmentedFlux = flux + noise
        augmentedFlux = normalise_spectrum(augmentedFlux)
        augmentedFlux = zero_non_overlap_part(augmentedFlux, minIndex, maxIndex, outerVal=0.5)

        return augmentedFlux


class OverSampling(ArrayTools):
    def __init__(self, nLabels, nw, **kwargs):
        """ Must take images and labels as arguments with the keyword specified.
        Can optionally take filenames and typeNames as arguments """
        ArrayTools.__init__(self, nLabels, nw)
        self.kwargs = kwargs

        counts = self.count_labels(self.kwargs['labels'])
        print("Before OverSample")  #
        print(counts)  #

        self.overSampleAmount = np.rint(div0(1 * max(counts), counts))  # ignore zeros in counts
        self.overSampleArraySize = int(sum(np.array(self.overSampleAmount, int) * counts))
        print(np.array(self.overSampleAmount, int) * counts)
        print(np.array(self.overSampleAmount, int))
        print(self.overSampleArraySize, len(self.kwargs['labels']))
        self.kwargOverSampled = {}
        self.randnum = np.random.randint(10000)
        for key in self.kwargs:
            if key == 'images':
                arrayOverSampled = np.memmap('oversampled_{}_{}.dat'.format(key, self.randnum), dtype=np.float16,
                                             mode='w+',
                                             shape=(self.overSampleArraySize, int(self.nw)))
            elif key == 'labels':
                arrayOverSampled = np.memmap('oversampled_{}_{}.dat'.format(key, self.randnum), dtype=np.uint16,
                                             mode='w+',
                                             shape=self.overSampleArraySize)
            else:
                arrayOverSampled = np.memmap('oversampled_{}_{}.dat'.format(key, self.randnum), dtype=object, mode='w+',
                                             shape=self.overSampleArraySize)
            self.kwargOverSampled[key] = arrayOverSampled

        self.kwargShuf = self.shuffle_arrays(memmapName='pre-oversample_{}'.format(self.randnum), **self.kwargs)
        print(len(self.kwargShuf['labels']))

    def oversample_mp(self, i_in, offset_in, std_in, labelIndex_in):
        print('oversampling', i_in, len(self.kwargShuf['labels']))
        oversampled = {key: [] for key in self.kwargs}
        repeatAmount = int(self.overSampleAmount[labelIndex_in])
        for r in range(repeatAmount):
            for key in self.kwargs:
                if key == 'images':
                    oversampled[key].append(
                        self.augment_data(self.kwargShuf[key][i_in], stdDevMean=0.05, stdDevStdDev=std_in))
                else:
                    oversampled[key].append(self.kwargShuf[key][i_in])
        return oversampled, offset_in, repeatAmount

    def collect_results(self, result):
        """Uses apply_async's callback to setup up a separate Queue for each process"""
        oversampled_in, offset_in, repeatAmount = result
        for key in self.kwargs:
            rlength_array = np.array(oversampled_in[key])
            self.kwargOverSampled[key][offset_in:repeatAmount + offset_in] = rlength_array[:]

    def over_sample_arrays(self, smote=False):
        if smote:
            return self.smote_oversample()
        else:
            return self.minority_oversample_with_noise()

    def minority_oversample_with_noise(self):
        offset = 0
        # pool = mp.Pool()
        for i in range(len(self.kwargShuf['labels'])):
            labelIndex = self.kwargShuf['labels'][i]
            if self.overSampleAmount[labelIndex] < 10:
                std = 0.03
            else:
                std = 0.05
            # pool.apply_async(self.oversample_mp, args=(i, offset, std, labelIndex), callback=self.collect_results)
            self.collect_results(self.oversample_mp(i, offset, std, labelIndex))
            offset += int(self.overSampleAmount[labelIndex])
        # pool.close()
        # pool.join()

        # for i, output in enumerate(outputs):
        #     self.collect_results(output)
        #     print('combining results...', i, len(outputs))

        print("Before Shuffling")
        self.kwargOverSampledShuf = self.shuffle_arrays(memmapName='oversampled_{}'.format(self.randnum),
                                                        **self.kwargOverSampled)
        print("After Shuffling")

        return self.kwargOverSampledShuf

    def smote_oversample(self):
        sm = over_sampling.SMOTE(random_state=42, n_jobs=30)
        images, labels = sm.fit_sample(X=self.kwargShuf['images'], y=self.kwargShuf['labels'])

        self.kwargOverSampledShuf = self.shuffle_arrays(memmapName='oversampled_smote_{}'.format(self.randnum),
                                                        images=images, labels=labels)

        return self.kwargOverSampledShuf


class CreateArrays(object):
    def __init__(self, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ, numOfRedshifts,
                 hostTypes=None, nHostTypes=None):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.minZ = minZ
        self.maxZ = maxZ
        self.numOfRedshifts = numOfRedshifts
        self.ageBinning = AgeBinning(minAge, maxAge, ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(maxAge - 0.1) + 1
        self.nLabels = nTypes * self.numOfAgeBins * nHostTypes
        self.createLabels = CreateLabels(self.nTypes, self.minAge, self.maxAge, self.ageBinSize, self.typeList,
                                         hostTypes, nHostTypes)
        self.hostTypes = hostTypes

    def combined_sn_gal_templates_to_arrays(self, args):
        print("enter func")
        snTemplateLocation, snTempList, galTemplateLocation, galTempList, snFractions = args
        ncolsDict = {'PTF10bzf.lnw': 2, 'PTF10qts.lnw': 6, 'PTF10vgv.lnw': 4, 'PTF12gzk.lnw': 8, 'iPTF13bvn.lnw': 16,
                     'sn00cn_bsnip.lnw': 1, 'sn00cp_bsnip.lnw': 1, 'sn00cu_bsnip.lnw': 2, 'sn00cw_bsnip.lnw': 1,
                     'sn00cx.lnw': 20, 'sn00dg_bsnip.lnw': 2, 'sn00dk_bsnip.lnw': 4, 'sn00dm_bsnip.lnw': 2,
                     'sn00dn_bsnip.lnw': 2, 'sn00fa_bsnip.lnw': 2, 'sn01bg_bsnip.lnw': 2, 'sn01dw_bsnip.lnw': 1,
                     'sn01eh_bsnip.lnw': 4, 'sn01en_bsnip.lnw': 3, 'sn01ep_bsnip.lnw': 5, 'sn02bg_bsnip.lnw': 1,
                     'sn02bo_bsnip.lnw': 5, 'sn02bz_bsnip.lnw': 1, 'sn02cf_bsnip.lnw': 1, 'sn02cr_bsnip.lnw': 3,
                     'sn02cs_bsnip.lnw': 2, 'sn02cx.lnw': 7, 'sn02ef_bsnip.lnw': 2, 'sn02el_bsnip.lnw': 2,
                     'sn02er.lnw': 23, 'sn02er_bsnip.lnw': 3, 'sn02eu_bsnip.lnw': 2, 'sn02fb_bsnip.lnw': 2,
                     'sn02fk_bsnip.lnw': 3, 'sn02ha_bsnip.lnw': 4, 'sn02hd_bsnip.lnw': 1, 'sn02he_bsnip.lnw': 4,
                     'sn02jy_bsnip.lnw': 2, 'sn02kf_bsnip.lnw': 1, 'sn03U_bsnip.lnw': 1, 'sn03Y_bsnip.lnw': 2,
                     'sn03ai_bsnip.lnw': 1, 'sn03cq_bsnip.lnw': 1, 'sn03gq_bsnip.lnw': 0, 'sn03he_bsnip.lnw': 2,
                     'sn03iv_bsnip.lnw': 2, 'sn04as_bsnip.lnw': 1, 'sn04bg_bsnip.lnw': 1, 'sn04bl_bsnip.lnw': 1,
                     'sn04bw_bsnip.lnw': 1, 'sn04dt_bsnip.lnw': 3, 'sn04ef_bsnip.lnw': 3, 'sn04eo_bsnip.lnw': 3,
                     'sn04ey_bsnip.lnw': 2, 'sn04fu_bsnip.lnw': 3, 'sn04fz_bsnip.lnw': 3, 'sn04gs_bsnip.lnw': 1,
                     'sn05am_bsnip.lnw': 3, 'sn05bc_bsnip.lnw': 2, 'sn05be_bsnip.lnw': 2, 'sn05bl_bsnip.lnw': 1,
                     'sn05cf_bsnip.lnw': 5, 'sn05de_bsnip.lnw': 4, 'sn05dv_bsnip.lnw': 1, 'sn05el_bsnip.lnw': 3,
                     'sn05eq_bsnip.lnw': 2, 'sn05gj_bsnip.lnw': 2, 'sn05hk_bsnip.lnw': 3, 'sn05kc_bsnip.lnw': 2,
                     'sn05ke_bsnip.lnw': 3, 'sn05ki_bsnip.lnw': 1, 'sn05ms_bsnip.lnw': 2, 'sn06N_bsnip.lnw': 4,
                     'sn06ac_bsnip.lnw': 1, 'sn06bq_bsnip.lnw': 3, 'sn06bu_bsnip.lnw': 1, 'sn06bz_bsnip.lnw': 1,
                     'sn06cf_bsnip.lnw': 2, 'sn06cp_bsnip.lnw': 1, 'sn06cq_bsnip.lnw': 1, 'sn06cs_bsnip.lnw': 1,
                     'sn06cz_bsnip.lnw': 1, 'sn06dm_bsnip.lnw': 4, 'sn06dw_bsnip.lnw': 4, 'sn06ef_bsnip.lnw': 3,
                     'sn06ej_bsnip.lnw': 4, 'sn06em_bsnip.lnw': 2, 'sn06et_bsnip.lnw': 3, 'sn06ev_bsnip.lnw': 2,
                     'sn06gt_bsnip.lnw': 1, 'sn06ke_bsnip.lnw': 1, 'sn06kf_bsnip.lnw': 3, 'sn06lf_bsnip.lnw': 3,
                     'sn06or_bsnip.lnw': 2, 'sn06os_bsnip.lnw': 2, 'sn06sr_bsnip.lnw': 2, 'sn07A_bsnip.lnw': 2,
                     'sn07af_bsnip.lnw': 3, 'sn07al_bsnip.lnw': 1, 'sn07ba_bsnip.lnw': 3, 'sn07bc_bsnip.lnw': 2,
                     'sn07bd_bsnip.lnw': 1, 'sn07bm_bsnip.lnw': 4, 'sn07fb_bsnip.lnw': 4, 'sn07fr_bsnip.lnw': 1,
                     'sn07fs_bsnip.lnw': 4, 'sn07gk_bsnip.lnw': 2, 'sn07kk_bsnip.lnw': 1, 'sn07qe_bsnip.lnw': 3,
                     'sn08A_bsnip.lnw': 1, 'sn08bt_bsnip.lnw': 1, 'sn08ds_bsnip.lnw': 4, 'sn08ec_bsnip.lnw': 5,
                     'sn08hs_bsnip.lnw': 1, 'sn1979C.lnw': 4, 'sn1980K.lnw': 6, 'sn1981B.lnw': 6, 'sn1983N.lnw': 2,
                     'sn1983V.lnw': 12, 'sn1984A.lnw': 9, 'sn1984L.lnw': 9, 'sn1986G.lnw': 19, 'sn1987A.lnw': 130,
                     'sn1989B.lnw': 23, 'sn1990B.lnw': 15, 'sn1990I.lnw': 4, 'sn1990N.lnw': 7, 'sn1990O.lnw': 6,
                     'sn1990U.lnw': 4, 'sn1991M.lnw': 4, 'sn1991T.lnw': 13, 'sn1991bg.lnw': 17, 'sn1992A.lnw': 12,
                     'sn1992H.lnw': 3, 'sn1992ar.lnw': 1, 'sn1993J.lnw': 35, 'sn1993ac.lnw': 1, 'sn1994D.lnw': 29,
                     'sn1994I.lnw': 32, 'sn1994M.lnw': 10, 'sn1994Q.lnw': 6, 'sn1994S.lnw': 5, 'sn1994T.lnw': 5,
                     'sn1994ae.lnw': 13, 'sn1995D.lnw': 9, 'sn1995E.lnw': 7, 'sn1995ac.lnw': 2, 'sn1995ak.lnw': 4,
                     'sn1995al.lnw': 11, 'sn1995bd.lnw': 7, 'sn1996C.lnw': 3, 'sn1996L.lnw': 3, 'sn1996X.lnw': 22,
                     'sn1996Z.lnw': 6, 'sn1996ab.lnw': 1, 'sn1996ai.lnw': 6, 'sn1996bk.lnw': 1, 'sn1996bl.lnw': 4,
                     'sn1996bo.lnw': 5, 'sn1996bv.lnw': 1, 'sn1996cb.lnw': 13, 'sn1997E.lnw': 4, 'sn1997Y.lnw': 4,
                     'sn1997bp.lnw': 11, 'sn1997bq.lnw': 13, 'sn1997br.lnw': 19, 'sn1997cn.lnw': 13, 'sn1997do.lnw': 14,
                     'sn1997dt.lnw': 7, 'sn1997ef.lnw': 24, 'sn1998S.lnw': 19, 'sn1998V.lnw': 9, 'sn1998ab.lnw': 12,
                     'sn1998aq.lnw': 17, 'sn1998bp.lnw': 11, 'sn1998bu.lnw': 26, 'sn1998bw.lnw': 19, 'sn1998co.lnw': 7,
                     'sn1998de.lnw': 7, 'sn1998dh.lnw': 10, 'sn1998dk.lnw': 9, 'sn1998dm.lnw': 10, 'sn1998dt.lnw': 9,
                     'sn1998dx.lnw': 7, 'sn1998ec.lnw': 6, 'sn1998ef.lnw': 4, 'sn1998eg.lnw': 5, 'sn1998es.lnw': 21,
                     'sn1998fa.lnw': 3, 'sn1999X.lnw': 6, 'sn1999aa.lnw': 35, 'sn1999ac.lnw': 31, 'sn1999aw.lnw': 4,
                     'sn1999bh.lnw': 1, 'sn1999by.lnw': 14, 'sn1999cc.lnw': 7, 'sn1999cl.lnw': 11, 'sn1999cp.lnw': 3,
                     'sn1999cw.lnw': 2, 'sn1999da.lnw': 1, 'sn1999dq.lnw': 19, 'sn1999ee.lnw': 12, 'sn1999ef.lnw': 3,
                     'sn1999ej.lnw': 5, 'sn1999ek.lnw': 4, 'sn1999em.lnw': 24, 'sn1999ex.lnw': 3, 'sn1999gd.lnw': 5,
                     'sn1999gh.lnw': 12, 'sn1999gi.lnw': 6, 'sn1999gp.lnw': 9, 'sn2000B.lnw': 5, 'sn2000E.lnw': 4,
                     'sn2000H.lnw': 11, 'sn2000bh.lnw': 1, 'sn2000bk.lnw': 3, 'sn2000ce.lnw': 4, 'sn2000cf.lnw': 6,
                     'sn2000cn.lnw': 9, 'sn2000cp.lnw': 3, 'sn2000cu.lnw': 1, 'sn2000cw.lnw': 1, 'sn2000cx.lnw': 26,
                     'sn2000dg.lnw': 4, 'sn2000dk.lnw': 6, 'sn2000dm.lnw': 1, 'sn2000dn.lnw': 4, 'sn2000er.lnw': 5,
                     'sn2000fa.lnw': 14, 'sn2001E.lnw': 3, 'sn2001G.lnw': 8, 'sn2001N.lnw': 8, 'sn2001V.lnw': 30,
                     'sn2001ah.lnw': 5, 'sn2001ay.lnw': 18, 'sn2001az.lnw': 5, 'sn2001bf.lnw': 6, 'sn2001bg.lnw': 5,
                     'sn2001br.lnw': 3, 'sn2001cj.lnw': 1, 'sn2001ck.lnw': 4, 'sn2001cp.lnw': 8, 'sn2001da.lnw': 6,
                     'sn2001eh.lnw': 18, 'sn2001el.lnw': 5, 'sn2001en.lnw': 9, 'sn2001ep.lnw': 22, 'sn2001ex.lnw': 1,
                     'sn2001fe.lnw': 9, 'sn2001fh.lnw': 6, 'sn2001gc.lnw': 8, 'sn2002G.lnw': 2, 'sn2002ao.lnw': 4,
                     'sn2002ap.lnw': 22, 'sn2002aw.lnw': 3, 'sn2002bf.lnw': 13, 'sn2002bo.lnw': 45, 'sn2002cd.lnw': 13,
                     'sn2002cf.lnw': 1, 'sn2002ck.lnw': 8, 'sn2002cr.lnw': 7, 'sn2002cs.lnw': 5, 'sn2002cu.lnw': 3,
                     'sn2002cx.lnw': 7, 'sn2002de.lnw': 10, 'sn2002dj.lnw': 20, 'sn2002dl.lnw': 4, 'sn2002do.lnw': 4,
                     'sn2002dp.lnw': 7, 'sn2002ef.lnw': 1, 'sn2002er.lnw': 25, 'sn2002es.lnw': 6, 'sn2002eu.lnw': 5,
                     'sn2002fb.lnw': 6, 'sn2002fk.lnw': 13, 'sn2002ha.lnw': 4, 'sn2002hd.lnw': 3, 'sn2002he.lnw': 5,
                     'sn2002hu.lnw': 4, 'sn2002hw.lnw': 5, 'sn2002ic.lnw': 4, 'sn2002jg.lnw': 4, 'sn2002jy.lnw': 10,
                     'sn2002kf.lnw': 11, 'sn2003U.lnw': 3, 'sn2003W.lnw': 16, 'sn2003Y.lnw': 3, 'sn2003bg.lnw': 8,
                     'sn2003cg.lnw': 40, 'sn2003ch.lnw': 12, 'sn2003cq.lnw': 7, 'sn2003dh.lnw': 10, 'sn2003du.lnw': 43,
                     'sn2003fa.lnw': 19, 'sn2003gn.lnw': 1, 'sn2003hu.lnw': 4, 'sn2003hv.lnw': 4, 'sn2003ic.lnw': 8,
                     'sn2003it.lnw': 15, 'sn2003iv.lnw': 7, 'sn2003jd.lnw': 19, 'sn2003kc.lnw': 7, 'sn2003kf.lnw': 21,
                     'sn2003lw.lnw': 2, 'sn2004L.lnw': 11, 'sn2004S.lnw': 7, 'sn2004as.lnw': 18, 'sn2004at.lnw': 11,
                     'sn2004aw.lnw': 25, 'sn2004bd.lnw': 4, 'sn2004bg.lnw': 5, 'sn2004bk.lnw': 1, 'sn2004dk.lnw': 3,
                     'sn2004dn.lnw': 3, 'sn2004dt.lnw': 32, 'sn2004ef.lnw': 15, 'sn2004eo.lnw': 14, 'sn2004et.lnw': 24,
                     'sn2004fe.lnw': 9, 'sn2004ff.lnw': 1, 'sn2004fu.lnw': 9, 'sn2004fz.lnw': 1, 'sn2004gc.lnw': 1,
                     'sn2004ge.lnw': 1, 'sn2004gq.lnw': 15, 'sn2004gs.lnw': 10, 'sn2004gt.lnw': 6, 'sn2004gv.lnw': 4,
                     'sn2005A.lnw': 3, 'sn2005M.lnw': 9, 'sn2005am.lnw': 16, 'sn2005az.lnw': 18, 'sn2005bc.lnw': 1,
                     'sn2005be.lnw': 1, 'sn2005bf.lnw': 30, 'sn2005bl.lnw': 8, 'sn2005bo.lnw': 2, 'sn2005cc.lnw': 10,
                     'sn2005cf.lnw': 61, 'sn2005cg.lnw': 6, 'sn2005cs.lnw': 28, 'sn2005ek.lnw': 6, 'sn2005el.lnw': 7,
                     'sn2005eq.lnw': 8, 'sn2005eu.lnw': 6, 'sn2005gj.lnw': 10, 'sn2005hc.lnw': 3, 'sn2005hf.lnw': 7,
                     'sn2005hg.lnw': 17, 'sn2005hj.lnw': 10, 'sn2005hk.lnw': 30, 'sn2005iq.lnw': 1, 'sn2005kc.lnw': 6,
                     'sn2005ke.lnw': 10, 'sn2005ki.lnw': 1, 'sn2005kl.lnw': 2, 'sn2005la.lnw': 5, 'sn2005ls.lnw': 2,
                     'sn2005lu.lnw': 3, 'sn2005mc.lnw': 2, 'sn2005mf.lnw': 4, 'sn2005mz.lnw': 4, 'sn2005na.lnw': 6,
                     'sn2006D.lnw': 6, 'sn2006H.lnw': 14, 'sn2006N.lnw': 12, 'sn2006S.lnw': 9, 'sn2006T.lnw': 5,
                     'sn2006X.lnw': 21, 'sn2006ac.lnw': 11, 'sn2006aj.lnw': 25, 'sn2006ak.lnw': 1, 'sn2006al.lnw': 6,
                     'sn2006ax.lnw': 4, 'sn2006az.lnw': 1, 'sn2006bp.lnw': 19, 'sn2006bq.lnw': 1, 'sn2006br.lnw': 2,
                     'sn2006bt.lnw': 13, 'sn2006bw.lnw': 1, 'sn2006bz.lnw': 3, 'sn2006cc.lnw': 1, 'sn2006cf.lnw': 6,
                     'sn2006cj.lnw': 8, 'sn2006cm.lnw': 2, 'sn2006cp.lnw': 10, 'sn2006cq.lnw': 1, 'sn2006cz.lnw': 3,
                     'sn2006el.lnw': 7, 'sn2006em.lnw': 1, 'sn2006ep.lnw': 5, 'sn2006eq.lnw': 1, 'sn2006et.lnw': 2,
                     'sn2006eu.lnw': 1, 'sn2006ev.lnw': 1, 'sn2006gj.lnw': 2, 'sn2006gr.lnw': 17, 'sn2006gt.lnw': 3,
                     'sn2006gz.lnw': 17, 'sn2006hb.lnw': 8, 'sn2006jc.lnw': 18, 'sn2006kf.lnw': 1, 'sn2006le.lnw': 22,
                     'sn2006lf.lnw': 19, 'sn2006mo.lnw': 1, 'sn2006nz.lnw': 1, 'sn2006oa.lnw': 7, 'sn2006ot.lnw': 2,
                     'sn2006sr.lnw': 6, 'sn2006te.lnw': 1, 'sn2007A.lnw': 3, 'sn2007C.lnw': 11, 'sn2007D.lnw': 1,
                     'sn2007F.lnw': 11, 'sn2007S.lnw': 10, 'sn2007Y.lnw': 8, 'sn2007ae.lnw': 2, 'sn2007af.lnw': 20,
                     'sn2007ag.lnw': 3, 'sn2007al.lnw': 8, 'sn2007ap.lnw': 2, 'sn2007au.lnw': 4, 'sn2007ax.lnw': 9,
                     'sn2007ba.lnw': 3, 'sn2007bc.lnw': 7, 'sn2007bd.lnw': 6, 'sn2007bg.lnw': 4, 'sn2007bj.lnw': 5,
                     'sn2007bm.lnw': 11, 'sn2007bz.lnw': 1, 'sn2007ca.lnw': 9, 'sn2007cg.lnw': 3, 'sn2007ci.lnw': 11,
                     'sn2007cl.lnw': 2, 'sn2007co.lnw': 16, 'sn2007cq.lnw': 2, 'sn2007fb.lnw': 6, 'sn2007fs.lnw': 4,
                     'sn2007gr.lnw': 15, 'sn2007hj.lnw': 14, 'sn2007if.lnw': 32, 'sn2007jg.lnw': 2, 'sn2007kj.lnw': 2,
                     'sn2007kk.lnw': 14, 'sn2007le.lnw': 23, 'sn2007nq.lnw': 2, 'sn2007qe.lnw': 10, 'sn2007ru.lnw': 9,
                     'sn2007sr.lnw': 7, 'sn2007ux.lnw': 2, 'sn2007uy.lnw': 7, 'sn2008A.lnw': 14, 'sn2008C.lnw': 4,
                     'sn2008D.lnw': 18, 'sn2008L.lnw': 1, 'sn2008Q.lnw': 6, 'sn2008R.lnw': 1, 'sn2008Z.lnw': 14,
                     'sn2008ae.lnw': 5, 'sn2008af.lnw': 2, 'sn2008ar.lnw': 16, 'sn2008ax.lnw': 22, 'sn2008bf.lnw': 12,
                     'sn2008bo.lnw': 9, 'sn2009bb.lnw': 11, 'sn2009er.lnw': 8, 'sn2009iz.lnw': 9, 'sn2009jf.lnw': 33,
                     'sn2009mg.lnw': 10, 'sn2009nz.lnw': 1, 'sn2010ay.lnw': 2, 'sn2010ma.lnw': 3, 'sn2011bm.lnw': 8,
                     'sn2011dh.lnw': 28, 'sn2011ei.lnw': 11, 'sn2011fu.lnw': 7, 'sn2012ap.lnw': 11, 'sn2012bz.lnw': 3,
                     'sn2013cq.lnw': 1, 'sn2013dk.lnw': 1, 'sn2013dx.lnw': 16, 'sn89B.lnw': 23, 'sn91T.lnw': 13,
                     'sn91T_bsnip.lnw': 3, 'sn91bg.lnw': 16, 'sn91bg_bsnip.lnw': 5, 'sn94D.lnw': 36, 'sn94I.lnw': 18,
                     'sn94S_bsnip.lnw': 1, 'sn94ae.lnw': 13, 'sn94ae_bsnip.lnw': 0, 'sn95D.lnw': 9,
                     'sn95D_bsnip.lnw': 2, 'sn95E_bsnip.lnw': 1, 'sn95ac.lnw': 1, 'sn95ac_bsnip.lnw': 1,
                     'sn97Y_bsnip.lnw': 1, 'sn97br_bsnip.lnw': 2, 'sn98bu.lnw': 27, 'sn98bu_bsnip.lnw': 1,
                     'sn98bw.lnw': 16, 'sn98dx_bsnip.lnw': 1, 'sn98ec_bsnip.lnw': 1, 'sn98ef_bsnip.lnw': 2,
                     'sn98es_bsnip.lnw': 2, 'sn99aa_bsnip.lnw': 4, 'sn99by.lnw': 14, 'sn99by_bsnip.lnw': 0,
                     'sn99cp_bsnip.lnw': 2, 'sn99da_bsnip.lnw': 3, 'sn99dk_bsnip.lnw': 4, 'sn99dq_bsnip.lnw': 3,
                     'sn99gd_bsnip.lnw': 1, 'snls03D3bb.lnw': 1}

        print("defined dict")
        numSpectra = 0
        for name in snTempList:
            try:
                numSpectra += ncolsDict[name]
            except KeyError:
                print(name, "NOT IN DICTIONARY")

        print("memmory mapped in func")
        randnum = np.random.randint(10000)
        arraySize = numSpectra * len(galTempList) * len(snFractions) * self.numOfRedshifts
        images = np.memmap('images_{}_{}.dat'.format(snTempList[0], randnum), dtype=np.float16, mode='w+',
                           shape=(arraySize, int(self.nw)))
        labelsIndexes = np.memmap('labels_{}_{}.dat'.format(snTempList[0], randnum), dtype=np.uint16, mode='w+',
                                  shape=arraySize)
        filenames = np.memmap('filenames_{}_{}.dat'.format(snTempList[0], randnum), dtype=object, mode='w+',
                              shape=arraySize)
        typeNames = np.memmap('typeNames_{}_{}.dat'.format(snTempList[0], randnum), dtype=object, mode='w+',
                              shape=arraySize)
        nRows = 0
        print("Create array loop begin...")
        for j in range(len(galTempList)):
            galFilename = galTemplateLocation + galTempList[j] if galTemplateLocation is not None else None
            for i in range(0, len(snTempList)):
                nCols = 15
                readSpectra = ReadSpectra(self.w0, self.w1, self.nw, snTemplateLocation + snTempList[i], galFilename)
                for ageidx in range(0, 1000):
                    if ageidx >= nCols:
                        break
                    for snCoeff in snFractions:
                        galCoeff = 1 - snCoeff
                        if self.numOfRedshifts == 1:
                            redshifts = [self.minZ]
                        else:
                            redshifts = np.random.uniform(low=self.minZ, high=self.maxZ, size=self.numOfRedshifts)
                        for z in redshifts:
                            tempWave, tempFlux, nCols, ages, tType, tMinIndex, tMaxIndex = readSpectra.sn_plus_gal_template(
                                ageidx, snCoeff, galCoeff, z)
                            if tMinIndex == tMaxIndex or not tempFlux.any():
                                print("NO DATA for {} {} ageIdx:{} z>={}".format(galTempList[j], snTempList[i], ageidx,
                                                                                 z))
                                break

                            if self.minAge < float(ages[ageidx]) < self.maxAge:
                                if self.hostTypes is None:  # Checks if we are classifying by host as well
                                    labelIndex, typeName = self.createLabels.label_array(tType, ages[ageidx], host=None)
                                else:
                                    labelIndex, typeName = self.createLabels.label_array(tType, ages[ageidx],
                                                                                         host=galTempList[j])
                                if tMinIndex > (self.nw - 1):
                                    continue
                                nonzeroflux = tempFlux[tMinIndex:tMaxIndex + 1]
                                newflux = (nonzeroflux - min(nonzeroflux)) / (max(nonzeroflux) - min(nonzeroflux))
                                newflux2 = np.concatenate((tempFlux[0:tMinIndex], newflux, tempFlux[tMaxIndex + 1:]))
                                images[nRows] = np.array([newflux2])
                                labelsIndexes[nRows] = labelIndex
                                filenames[nRows] = "{0}_{1}_{2}_{3}_snCoeff{4}_z{5}".format(snTempList[i], tType,
                                                                                            str(ages[ageidx]),
                                                                                            galTempList[j], snCoeff,
                                                                                            (z))
                                typeNames[nRows] = typeName
                                nRows += 1
                print(snTempList[i], nCols, galTempList[j])

        print("Done func")
        return images, labelsIndexes, filenames, typeNames, nRows

    def collect_results(self, result, wheretostartappending):
        """Uses apply_async's callback to setup up a separate Queue for each process"""
        imagesPart, labelsPart, filenamesPart, typeNamesPart, nRows = result

        self.images[wheretostartappending:wheretostartappending + nRows] = imagesPart[0:nRows]
        self.labelsIndexes[wheretostartappending:wheretostartappending + nRows] = labelsPart[0:nRows]
        self.filenames[wheretostartappending:wheretostartappending + nRows] = filenamesPart[0:nRows]
        self.typeNames[wheretostartappending:wheretostartappending + nRows] = typeNamesPart[0:nRows]

    def combined_sn_gal_arrays_multiprocessing(self, snTemplateLocation, snTempFileList, galTemplateLocation,
                                               galTempFileList):
        if galTemplateLocation is None or galTempFileList is None:
            galTempList = [None]
            galTemplateLocation = None
            snFractions = [1.0]
        else:
            galTempList = temp_list(galTempFileList)
            snFractions = [0.99, 0.98, 0.95, 0.93, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1]

        snTempList = temp_list(snTempFileList)  # 514 files
        galAndSnTemps = list(itertools.product(galTempList, snTempList))[0:2]
        argsList = []
        for gal, sn in galAndSnTemps:
            argsList.append((snTemplateLocation, [sn], galTemplateLocation, [gal], snFractions))

        numSpectra = 3968  # sum nColsDict
        arraySize = numSpectra * len(galTempList) * len(snFractions) * self.numOfRedshifts
        print("Arraysize is:", arraySize)

        self.images = np.memmap('all_images.dat', dtype=np.float16, mode='w+', shape=(arraySize, int(self.nw)))
        self.labelsIndexes = np.memmap('all_labels.dat', dtype=np.uint16, mode='w+', shape=arraySize)
        self.filenames = np.memmap('all_filenames.dat', dtype=object, mode='w+', shape=arraySize)
        self.typeNames = np.memmap('all_typeNames.dat', dtype=object, mode='w+', shape=arraySize)

        print("images GiB:", self.images.nbytes / 2 ** 30)
        print("labels GiB:", self.labelsIndexes.nbytes / 2 ** 30)
        print("filenames GiB:", self.filenames.nbytes / 2 ** 30)
        print("typeNames GiB:", self.typeNames.nbytes / 2 ** 30)

        # #  Multiprocessing with map_async (faster)
        # pool = mp.Pool()
        # results = pool.map_async(self.combined_sn_gal_templates_to_arrays, argsList)
        # pool.close()
        # pool.join()
        # outputs = results.get()
        # for i, output in enumerate(outputs):
        #     self.collect_results(output)
        #     print('combining results...', output[-1], i, len(outputs))

        print("Begin pooling...")
        # #  Multiprocessing with apply_async (better when arrays are large - i.e. agnostic redshift)
        results = []
        pool = mp.Pool(processes=50)
        print("pool")
        for arg in argsList:
            print("argLoop")
            result = pool.apply_async(self.combined_sn_gal_templates_to_arrays, [arg])
            results.append(result)
        print("close pool")
        pool.close()
        pool.join()
        print("Finished Pooling")

        wheretostartappending = 0
        for i, p in enumerate(results):
            output = p.get()
            nRows = output[-1]
            self.collect_results(output, wheretostartappending)
            wheretostartappending += nRows
            print('combining results...', nRows, i, len(results))

        print("Completed Creating Arrays!")

        # Delete temporary memory mapping files
        for filename in glob.glob('images_*.dat'):
            os.remove(filename)
        for filename in glob.glob('labels_*.dat'):
            os.remove(filename)
        for filename in glob.glob('filenames_*.dat'):
            os.remove(filename)
        for filename in glob.glob('typeNames_*.dat'):
            os.remove(filename)

        return self.images, self.labelsIndexes, self.filenames, self.typeNames



import os
import subprocess
import astrodash

directoryPath = os.path.join(os.path.dirname(os.path.abspath(__file__)), '../templates/run30/')

allFilePaths = []
for dirpath, dirnames, filenames in os.walk(directoryPath):
    for filename in [f for f in filenames if f.endswith(".dat")]:
        allFilePaths.append(os.path.join(dirpath, filename))

run30 = [('DES17E2cc', 0.149),
         ('DES17C3de', 0.107),
         ('DES17C3eg', 0.117),
         ('DES17X3az', 0.56),
         ('DES17X3bd', 0.141),
         ('DES17X3ca', 0.198),
         ('DES17X3cb', 0.317),
         ('DES17X3ct', 0.206),
         ('DES17E2a', 0.295),
         ('DES17E2b', 0.227),
         ('DES17E2aq', 0.352),
         ('DES17E2ar', 0.513),
         ('DES17E2bw', 0.147),
         ('DES17E2bx', 0.272),
         ('DES17E1by', 0.287),
         ('DES17E2ce', 0.269),
         ('DES17E2ci', 0.127),

         ('DES17E2f', 0.586),
         ('DES17E2ay', 0.186),
         ('DES17E1bp', 0.46),
         ('DES17E1ck', 0.221),
         ('DES17C3ef', 0.161),
         ('DES17E1by', 0.287),
         ('DES17C3ed', 0.354),
         ]

names = [i[0] for i in run30]
knownRedshifts = [i[1] for i in run30]

allFilePaths.reverse()
run30 = []

for i in range(len(names)):
    for filePath in allFilePaths:
        if names[i] == filePath.split('/')[-1].split('_')[0]:
            run30.append((filePath, knownRedshifts[i]))
            break

filenames = [i[0] for i in run30]
knownRedshifts = [i[1] for i in run30]

classification = astrodash.Classify(filenames, knownRedshifts, classifyHost=False, rlapScores=True, smooth=6)
bestFits, redshifts, bestTypes, rejectionLabels, reliableFlags, redshiftErrs = classification.list_best_matches(n=5, saveFilename='DASH_matches_run30_new.txt')
classification.plot_with_gui(indexToPlot=22)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'design.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1205, 692)
        MainWindow.setWindowOpacity(1.0)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.verticalLayout.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_2.setMaximumSize(QtCore.QSize(16777215, 150))
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.btnBrowse = QtWidgets.QPushButton(self.groupBox_2)
        self.btnBrowse.setMinimumSize(QtCore.QSize(100, 0))
        self.btnBrowse.setMaximumSize(QtCore.QSize(100, 50))
        self.btnBrowse.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.btnBrowse.setObjectName("btnBrowse")
        self.gridLayout_2.addWidget(self.btnBrowse, 1, 0, 1, 1, QtCore.Qt.AlignHCenter)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem = QtWidgets.QSpacerItem(25, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem)
        self.lineEditInputFilename = QtWidgets.QLineEdit(self.groupBox_2)
        self.lineEditInputFilename.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setItalic(False)
        font.setUnderline(False)
        font.setWeight(50)
        font.setKerning(True)
        self.lineEditInputFilename.setFont(font)
        self.lineEditInputFilename.setAutoFillBackground(False)
        self.lineEditInputFilename.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEditInputFilename.setReadOnly(False)
        self.lineEditInputFilename.setObjectName("lineEditInputFilename")
        self.horizontalLayout_8.addWidget(self.lineEditInputFilename)
        spacerItem1 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem1)
        self.gridLayout_2.addLayout(self.horizontalLayout_8, 0, 0, 1, 1)
        self.verticalLayout_7.addLayout(self.gridLayout_2)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox.setMaximumSize(QtCore.QSize(16777215, 390))
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setMaximumSize(QtCore.QSize(16777215, 240))
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 223, 198))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setContentsMargins(-1, -1, -1, 5)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.checkBoxKnownZ = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxKnownZ.setChecked(True)
        self.checkBoxKnownZ.setObjectName("checkBoxKnownZ")
        self.horizontalLayout_13.addWidget(self.checkBoxKnownZ)
        self.lineEditKnownZ = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditKnownZ.setObjectName("lineEditKnownZ")
        self.horizontalLayout_13.addWidget(self.lineEditKnownZ)
        self.horizontalLayout_13.setStretch(0, 2)
        self.horizontalLayout_13.setStretch(1, 1)
        self.verticalLayout_5.addLayout(self.horizontalLayout_13)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.checkBoxClassifyHost = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxClassifyHost.setObjectName("checkBoxClassifyHost")
        self.horizontalLayout.addWidget(self.checkBoxClassifyHost)
        self.verticalLayout_5.addLayout(self.horizontalLayout)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setContentsMargins(-1, 0, -1, -1)
        self.gridLayout.setObjectName("gridLayout")
        self.label_8 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 1, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 0, 0, 1, 1)
        self.lineEditMinWave = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditMinWave.setObjectName("lineEditMinWave")
        self.gridLayout.addWidget(self.lineEditMinWave, 0, 1, 1, 1)
        self.lineEditMaxWave = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditMaxWave.setObjectName("lineEditMaxWave")
        self.gridLayout.addWidget(self.lineEditMaxWave, 1, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 0, 2, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 1, 2, 1, 1)
        self.gridLayout.setColumnStretch(0, 1)
        self.gridLayout.setColumnStretch(1, 1)
        self.gridLayout.setColumnStretch(2, 1)
        self.verticalLayout_5.addLayout(self.gridLayout)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_3 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.lineEditSmooth = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditSmooth.setMaximumSize(QtCore.QSize(40, 16777215))
        self.lineEditSmooth.setObjectName("lineEditSmooth")
        self.horizontalLayout_4.addWidget(self.lineEditSmooth)
        self.horizontalSliderSmooth = QtWidgets.QSlider(self.scrollAreaWidgetContents)
        self.horizontalSliderSmooth.setMaximum(20)
        self.horizontalSliderSmooth.setProperty("value", 6)
        self.horizontalSliderSmooth.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderSmooth.setObjectName("horizontalSliderSmooth")
        self.horizontalLayout_4.addWidget(self.horizontalSliderSmooth)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(-1, -1, -1, 10)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.checkBoxRlap = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxRlap.setObjectName("checkBoxRlap")
        self.verticalLayout_3.addWidget(self.checkBoxRlap)
        self.verticalLayout_5.addLayout(self.verticalLayout_3)
        self.verticalLayout_5.setStretch(0, 1)
        self.verticalLayout_5.setStretch(1, 1)
        self.verticalLayout_5.setStretch(2, 2)
        self.verticalLayout_5.setStretch(3, 1)
        self.verticalLayout_4.addLayout(self.verticalLayout_5)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_6.addWidget(self.scrollArea)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.btnRefit = QtWidgets.QPushButton(self.groupBox)
        self.btnRefit.setMaximumSize(QtCore.QSize(150, 16777215))
        self.btnRefit.setObjectName("btnRefit")
        self.horizontalLayout_6.addWidget(self.btnRefit)
        self.verticalLayout_6.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.btnCancel = QtWidgets.QPushButton(self.groupBox)
        self.btnCancel.setMinimumSize(QtCore.QSize(0, 0))
        self.btnCancel.setMaximumSize(QtCore.QSize(100, 16777215))
        self.btnCancel.setObjectName("btnCancel")
        self.horizontalLayout_7.addWidget(self.btnCancel)
        self.verticalLayout_6.addLayout(self.horizontalLayout_7)
        self.verticalLayout.addWidget(self.groupBox)
        self.progressBar = QtWidgets.QProgressBar(self.centralwidget)
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout.addWidget(self.progressBar)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.btnQuit = QtWidgets.QPushButton(self.centralwidget)
        self.btnQuit.setMaximumSize(QtCore.QSize(120, 16777215))
        self.btnQuit.setObjectName("btnQuit")
        self.horizontalLayout_15.addWidget(self.btnQuit)
        self.verticalLayout.addLayout(self.horizontalLayout_15)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem4)
        self.verticalLayout.setStretch(0, 1)
        self.verticalLayout.setStretch(1, 6)
        self.verticalLayout.setStretch(4, 2)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_2.setContentsMargins(-1, -1, 0, -1)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox_3 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.listWidget = QtWidgets.QListWidget(self.groupBox_3)
        self.listWidget.setObjectName("listWidget")
        item = QtWidgets.QListWidgetItem()
        self.listWidget.addItem(item)
        self.horizontalLayout_5.addWidget(self.listWidget)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setContentsMargins(-1, -1, 0, 0)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.label_6 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily(".SF NS Text")
        font.setPointSize(13)
        font.setBold(True)
        font.setUnderline(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_10.addWidget(self.label_6)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.labelBestHostType = QtWidgets.QLabel(self.groupBox_3)
        self.labelBestHostType.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.labelBestHostType.setScaledContents(False)
        self.labelBestHostType.setAlignment(QtCore.Qt.AlignCenter)
        self.labelBestHostType.setWordWrap(False)
        self.labelBestHostType.setObjectName("labelBestHostType")
        self.horizontalLayout_10.addWidget(self.labelBestHostType)
        self.labelBestSnType = QtWidgets.QLabel(self.groupBox_3)
        self.labelBestSnType.setAlignment(QtCore.Qt.AlignCenter)
        self.labelBestSnType.setObjectName("labelBestSnType")
        self.horizontalLayout_10.addWidget(self.labelBestSnType)
        self.labelBestAgeRange = QtWidgets.QLabel(self.groupBox_3)
        self.labelBestAgeRange.setAlignment(QtCore.Qt.AlignCenter)
        self.labelBestAgeRange.setObjectName("labelBestAgeRange")
        self.horizontalLayout_10.addWidget(self.labelBestAgeRange)
        self.verticalLayout_10.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.horizontalLayout_14.addWidget(self.label)
        self.labelBestRedshift = QtWidgets.QLabel(self.groupBox_3)
        self.labelBestRedshift.setObjectName("labelBestRedshift")
        self.horizontalLayout_14.addWidget(self.labelBestRedshift)
        self.verticalLayout_10.addLayout(self.horizontalLayout_14)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_2 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_11.addWidget(self.label_2)
        self.labelBestRelProb = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(13)
        font.setBold(False)
        font.setWeight(50)
        self.labelBestRelProb.setFont(font)
        self.labelBestRelProb.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.labelBestRelProb.setObjectName("labelBestRelProb")
        self.horizontalLayout_11.addWidget(self.labelBestRelProb)
        self.verticalLayout_10.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.labelRlapWarning = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.labelRlapWarning.setFont(font)
        self.labelRlapWarning.setAlignment(QtCore.Qt.AlignCenter)
        self.labelRlapWarning.setObjectName("labelRlapWarning")
        self.horizontalLayout_12.addWidget(self.labelRlapWarning)
        self.labelInconsistentWarning = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.labelInconsistentWarning.setFont(font)
        self.labelInconsistentWarning.setAlignment(QtCore.Qt.AlignCenter)
        self.labelInconsistentWarning.setObjectName("labelInconsistentWarning")
        self.horizontalLayout_12.addWidget(self.labelInconsistentWarning)
        self.verticalLayout_10.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_5.addLayout(self.verticalLayout_10)
        self.verticalLayout_8.addLayout(self.horizontalLayout_5)
        self.verticalLayout_2.addWidget(self.groupBox_3)
        self.groupBox_4 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_4.setObjectName("groupBox_4")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_9.setContentsMargins(-1, -1, -1, 5)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_9 = QtWidgets.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_9.addWidget(self.label_9)
        self.comboBoxSNType = QtWidgets.QComboBox(self.groupBox_4)
        self.comboBoxSNType.setEditable(False)
        self.comboBoxSNType.setObjectName("comboBoxSNType")
        self.horizontalLayout_9.addWidget(self.comboBoxSNType)
        self.comboBoxAge = QtWidgets.QComboBox(self.groupBox_4)
        self.comboBoxAge.setObjectName("comboBoxAge")
        self.horizontalLayout_9.addWidget(self.comboBoxAge)
        self.comboBoxHost = QtWidgets.QComboBox(self.groupBox_4)
        self.comboBoxHost.setObjectName("comboBoxHost")
        self.horizontalLayout_9.addWidget(self.comboBoxHost)
        self.label_5 = QtWidgets.QLabel(self.groupBox_4)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_9.addWidget(self.label_5)
        self.lineEditHostFraction = QtWidgets.QLineEdit(self.groupBox_4)
        self.lineEditHostFraction.setObjectName("lineEditHostFraction")
        self.horizontalLayout_9.addWidget(self.lineEditHostFraction)
        self.horizontalSliderHostFraction = QtWidgets.QSlider(self.groupBox_4)
        self.horizontalSliderHostFraction.setMaximum(100)
        self.horizontalSliderHostFraction.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderHostFraction.setObjectName("horizontalSliderHostFraction")
        self.horizontalLayout_9.addWidget(self.horizontalSliderHostFraction)
        self.horizontalLayout_9.setStretch(0, 2)
        self.horizontalLayout_9.setStretch(1, 10)
        self.horizontalLayout_9.setStretch(2, 10)
        self.horizontalLayout_9.setStretch(3, 10)
        self.horizontalLayout_9.setStretch(4, 1)
        self.horizontalLayout_9.setStretch(5, 2)
        self.horizontalLayout_9.setStretch(6, 3)
        self.verticalLayout_9.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(-1, 3, -1, -1)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.labelTemplateName = QtWidgets.QLabel(self.groupBox_4)
        self.labelTemplateName.setMinimumSize(QtCore.QSize(270, 0))
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setItalic(True)
        self.labelTemplateName.setFont(font)
        self.labelTemplateName.setObjectName("labelTemplateName")
        self.horizontalLayout_2.addWidget(self.labelTemplateName)
        self.labelRlapScore = QtWidgets.QLabel(self.groupBox_4)
        self.labelRlapScore.setObjectName("labelRlapScore")
        self.horizontalLayout_2.addWidget(self.labelRlapScore)
        self.pushButtonLeftTemplate = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButtonLeftTemplate.setObjectName("pushButtonLeftTemplate")
        self.horizontalLayout_2.addWidget(self.pushButtonLeftTemplate)
        self.pushButtonRightTemplate = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButtonRightTemplate.setObjectName("pushButtonRightTemplate")
        self.horizontalLayout_2.addWidget(self.pushButtonRightTemplate)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.label_4 = QtWidgets.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.lineEditRedshift = QtWidgets.QLineEdit(self.groupBox_4)
        self.lineEditRedshift.setObjectName("lineEditRedshift")
        self.horizontalLayout_2.addWidget(self.lineEditRedshift)
        self.horizontalSliderRedshift = QtWidgets.QSlider(self.groupBox_4)
        self.horizontalSliderRedshift.setMaximum(10000)
        self.horizontalSliderRedshift.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderRedshift.setObjectName("horizontalSliderRedshift")
        self.horizontalLayout_2.addWidget(self.horizontalSliderRedshift)
        self.horizontalLayout_2.setStretch(6, 1)
        self.horizontalLayout_2.setStretch(7, 6)
        self.verticalLayout_9.addLayout(self.horizontalLayout_2)
        self.graphicsView_2 = PlotWidget(self.groupBox_4)
        self.graphicsView_2.setObjectName("graphicsView_2")
        self.verticalLayout_9.addWidget(self.graphicsView_2)
        self.graphicsView = PlotWidget(self.groupBox_4)
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout_9.addWidget(self.graphicsView)
        self.verticalLayout_9.setStretch(2, 1)
        self.verticalLayout_9.setStretch(3, 5)
        self.verticalLayout_2.addWidget(self.groupBox_4)
        self.verticalLayout_2.setStretch(0, 3)
        self.verticalLayout_2.setStretch(1, 11)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.horizontalLayout_3.setStretch(0, 9)
        self.horizontalLayout_3.setStretch(1, 32)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1205, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "DASH"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Select Spectrum"))
        self.btnBrowse.setText(_translate("MainWindow", "Browse"))
        self.lineEditInputFilename.setText(_translate("MainWindow", "Select SN File..."))
        self.groupBox.setTitle(_translate("MainWindow", "Priors"))
        self.checkBoxKnownZ.setText(_translate("MainWindow", "Known Redshift"))
        self.lineEditKnownZ.setText(_translate("MainWindow", "0"))
        self.checkBoxClassifyHost.setText(_translate("MainWindow", "Classify Host"))
        self.label_8.setText(_translate("MainWindow", "Max wave"))
        self.label_7.setText(_translate("MainWindow", "Min wave"))
        self.lineEditMinWave.setText(_translate("MainWindow", "3000"))
        self.lineEditMaxWave.setText(_translate("MainWindow", "10000"))
        self.label_3.setText(_translate("MainWindow", "Smooth"))
        self.lineEditSmooth.setText(_translate("MainWindow", "6"))
        self.checkBoxRlap.setText(_translate("MainWindow", "Calculate rlap scores"))
        self.btnRefit.setText(_translate("MainWindow", "Fit with priors"))
        self.btnCancel.setText(_translate("MainWindow", "Cancel"))
        self.btnQuit.setText(_translate("MainWindow", "Quit"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Best Matches"))
        __sortingEnabled = self.listWidget.isSortingEnabled()
        self.listWidget.setSortingEnabled(False)
        item = self.listWidget.item(0)
        item.setText(_translate("MainWindow", "Best Matches"))
        self.listWidget.setSortingEnabled(__sortingEnabled)
        self.label_6.setText(_translate("MainWindow", "Best Match"))
        self.labelBestHostType.setText(_translate("MainWindow", "Host Type"))
        self.labelBestSnType.setText(_translate("MainWindow", "SN Type"))
        self.labelBestAgeRange.setText(_translate("MainWindow", "Age Range"))
        self.label.setText(_translate("MainWindow", "Redshift:"))
        self.labelBestRedshift.setText(_translate("MainWindow", "0.0"))
        self.label_2.setText(_translate("MainWindow", "Softmax:"))
        self.labelBestRelProb.setText(_translate("MainWindow", "Rel. Prob."))
        self.labelRlapWarning.setText(_translate("MainWindow", "Warning"))
        self.labelInconsistentWarning.setText(_translate("MainWindow", "Labels"))
        self.groupBox_4.setTitle(_translate("MainWindow", "Analyse selection"))
        self.label_9.setText(_translate("MainWindow", "Plot Template"))
        self.label_5.setText(_translate("MainWindow", "Host Fraction"))
        self.lineEditHostFraction.setText(_translate("MainWindow", "0%"))
        self.labelTemplateName.setText(_translate("MainWindow", "Template Name"))
        self.labelRlapScore.setText(_translate("MainWindow", "rlap: "))
        self.pushButtonLeftTemplate.setText(_translate("MainWindow", "<"))
        self.pushButtonRightTemplate.setText(_translate("MainWindow", ">"))
        self.label_4.setText(_translate("MainWindow", "Redshift"))
        self.lineEditRedshift.setText(_translate("MainWindow", "0"))

from pyqtgraph import PlotWidget

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())




import os
import pickle
import tensorflow as tf
from astrodash.input_spectra import *
from astrodash.multilayer_convnet import convnet_variables


def get_training_parameters(data_files='models_v06'):
    scriptDirectory = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(scriptDirectory, data_files, "models/zeroZ/training_params.pickle"), 'rb') as f:
        pars = pickle.load(f)
    return pars


class LoadInputSpectra(object):
    def __init__(self, inputFilename, z, smooth, pars, minWave, maxWave, classifyHost):
        self.nw = pars['nw']
        nTypes, w0, w1, minAge, maxAge, ageBinSize, typeList = pars['nTypes'], pars['w0'], pars['w1'], pars['minAge'], \
                                                               pars['maxAge'], pars['ageBinSize'], pars['typeList']

        if classifyHost:
            hostList = pars['galTypeList']
            nHostTypes = len(hostList)
        else:
            hostList, nHostTypes = None, 1

        self.inputSpectra = InputSpectra(inputFilename, z, nTypes, minAge, maxAge, ageBinSize, w0, w1, self.nw,
                                         typeList, smooth, minWave, maxWave, hostList, nHostTypes)

        self.inputImages, self.inputFilenames, self.inputRedshifts, self.typeNamesList, self.inputMinMaxIndexes = self.inputSpectra.redshifting()
        self.nBins = len(self.typeNamesList)

    def input_spectra(self):
        return self.inputImages, self.inputRedshifts, self.typeNamesList, int(
            self.nw), self.nBins, self.inputMinMaxIndexes


class RestoreModel(object):
    def __init__(self, modelFilename, inputImages, nw, nBins):
        self.reset()

        self.modelFilename = modelFilename
        self.inputImages = inputImages
        self.nw = nw
        self.nBins = nBins
        self.imWidthReduc = 8
        self.imWidth = 32  # Image size and width

        self.x, self.y_, self.keep_prob, self.y_conv, self.W, self.b = convnet_variables(self.imWidth,
                                                                                         self.imWidthReduc, self.nw,
                                                                                         self.nBins)

        self.saver = tf.train.Saver()

    def restore_variables(self):
        with tf.Session() as sess:
            self.saver.restore(sess, self.modelFilename)

            softmax = self.y_conv.eval(feed_dict={self.x: self.inputImages, self.keep_prob: 1.0})
            # print(softmax)

            # dwlog = np.log(10000. / 3500.) / self.nw
            # wlog = 3500. * np.exp(np.arange(0, self.nw) * dwlog)
            #
            # import matplotlib.pyplot as plt
            # spec = np.loadtxt('spec91Tmax_2006cz.txt')
            # weight = sess.run(self.W)[:, 23]
            # # plt.scatter(np.arange(1024), self.inputImages[0], marker='.', c=weight, cmap=plt.get_cmap('seismic'))
            # plt.scatter(wlog, spec, marker='.', c=weight, cmap=plt.get_cmap('seismic'))
            # plt.colorbar()

            # plt.imshow(sess.run(self.W)[:, 5].reshape([32,32]), cmap=plt.get_cmap('seismic'))
            # for i, classval in enumerate(np.arange(5, 306, 18)):
            #     plt.subplot(9, 2, i + 1)
            #     weight = sess.run(self.W)[:, classval]
            #     plt.title("{}, {}".format(i, classval))
            #     plt.imshow(weight.reshape([32, 32]), cmap=plt.get_cmap('seismic'))
            #     frame1 = plt.gca()
            #     frame1.axes.get_xaxis().set_visible(False)
            #     frame1.axes.get_yaxis().set_visible(False)
        return softmax

    def reset(self):
        tf.reset_default_graph()


class BestTypesListSingleRedshift(object):
    def __init__(self, modelFilename, inputImages, typeNamesList, nw, nBins):
        self.modelFilename = modelFilename
        self.inputImages = inputImages
        self.typeNamesList = typeNamesList
        self.nBins = nBins

        self.restoreModel = RestoreModel(self.modelFilename, self.inputImages, nw, nBins)
        self.typeNamesList = np.array(typeNamesList)

        # if more than one image, then variables will be a list of length len(inputImages)
        softmaxes = self.restoreModel.restore_variables()
        self.bestTypes, self.softmaxOrdered, self.idx = [], [], []
        for softmax in softmaxes:
            bestTypes, idx, softmaxOrdered = self.create_list(softmax)
            self.bestTypes.append(bestTypes)
            self.softmaxOrdered.append(softmaxOrdered)
            self.idx.append(idx)

    def create_list(self, softmax):
        idx = np.argsort(softmax)  # list of the index of the highest probabiites
        bestTypes = self.typeNamesList[idx[::-1]]  # reordered in terms of softmax probability columns

        return bestTypes, idx, softmax[idx[::-1]]


def classification_split(classificationString):
    classification = classificationString.split(': ')
    if len(classification) == 2:
        snName, snAge = classification
        host = ""
    else:
        host, snName, snAge = classification

    return host, snName, snAge



import os
import sys
import numpy as np
import astropy.io.fits as afits
from scipy.interpolate import interp1d, UnivariateSpline
from astrodash.array_tools import normalise_spectrum, zero_non_overlap_part
from astrodash.read_from_catalog import catalogDict

try:
    import pandas as pd

    USE_PANDAS = True
except ImportError:
    print("Pandas module not installed. DASH will use numpy to load spectral files instead. "
          "This can be up to 10x slower.")
    USE_PANDAS = False


class ProcessingTools(object):
    def redshift_spectrum(self, wave, flux, z):
        wave_new = wave * (z + 1)

        return wave_new, flux

    def deredshift_spectrum(self, wave, flux, z):
        wave_new = wave / (z + 1)

        return wave_new, flux

    def min_max_index(self, flux, outerVal=0):
        """ 
        :param flux: 
        :param outerVal: is the scalar value in all entries before the minimum and after the maximum index
        :return: 
        """
        nonZeros = np.where(flux != outerVal)[0]
        if nonZeros.size:
            minIndex, maxIndex = min(nonZeros), max(nonZeros)
        else:
            minIndex, maxIndex = len(flux), len(flux)

        return minIndex, maxIndex


class ReadSpectrumFile(object):
    def __init__(self, filename, w0, w1, nw):
        self.filename = filename
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.processingTools = ProcessingTools()

    def read_fits_file(self):
        # filename = unicode(self.filename.toUtf8(), encoding="UTF-8")
        try:
            if 'specutils' not in sys.modules:
                from specutils.io import read_fits

            spectrum = read_fits.read_fits_spectrum1d(self.filename)
            if len(spectrum) > 1:
                spectrum = spectrum[0]
            try:
                wave = np.array(spectrum.wavelength)
                flux = np.array(spectrum.flux)
            except AttributeError:
                wave = np.array(spectrum.dispersion)
                flux = np.array(spectrum.flux)
                print("No wavelength attribute in FITS File. Using 'dispersion' attribute instead")
        except Exception as e:
            hdulist = afits.open(self.filename)
            flux = hdulist[0].data
            wave_start = hdulist[0].header['CRVAL1']
            if 'CD1_1' in hdulist[0].header:
                wave_step = hdulist[0].header['CD1_1']
            else:
                wave_step = hdulist[0].header['CDELT1']
            wave_num = flux.shape[0]
            wave = np.linspace(wave_start, wave_start + wave_step * wave_num, num=wave_num)

        flux[np.isnan(flux)] = 0  # convert nan's to zeros

        return wave, flux

    def read_dat_file(self):
        try:
            if USE_PANDAS is True:
                data = pd.read_csv(self.filename, header=None, delim_whitespace=True).values
            else:
                data = np.loadtxt(self.filename)
            wave = data[:, 0]
            flux = data[:, 1]
        except:
            print("COULDN'T USE LOADTXT FOR FILE: {0}\n READ LINE BY LINE INSTEAD.".format(self.filename))
            wave = []
            flux = []
            with open(self.filename, 'r') as FileObj:
                for line in FileObj:
                    if line.strip() != '' and line.strip()[0] != '#':
                        datapoint = line.rstrip('\n').strip().split()
                        wave.append(float(datapoint[0].replace('D', 'E')))
                        flux.append(float(datapoint[1].replace('D', 'E')))

            wave = np.array(wave)
            flux = np.array(flux)

        sorted_indexes = np.argsort(wave)
        wave = wave[sorted_indexes]
        flux = flux[sorted_indexes]

        return wave, flux

    def read_superfit_template(self):
        wave, flux = self.read_dat_file()
        tType = os.path.split(os.path.split(self.filename)[0])[-1]  # Name of directory is the type name
        filename = os.path.basename(self.filename)
        snName, ageInfo = os.path.basename(filename).strip('.dat').split('.')
        if ageInfo == 'max':
            age = 0
        elif ageInfo[0] == 'm':
            age = -float(ageInfo[1:])
        elif ageInfo[0] == 'p':
            age = float(ageInfo[1:])
        else:
            raise Exception("Invalid Superfit file: {0}".format(self.filename))

        nCols = 1

        return wave, flux, nCols, [age], tType

    def file_extension(self, template=False):
        if isinstance(self.filename, (list, np.ndarray)):  # Is an Nx2 array
            wave, flux = self.filename[0], self.filename[1]
            return wave, flux
        elif hasattr(self.filename, 'read'):  # Is a file handle
            self.filename.seek(0)
            return self.read_dat_file()
        else:  # Is a filename string
            filename = os.path.basename(self.filename)
            extension = filename.split('.')[-1]

            if template is True and extension == 'dat' and len(filename.split('.')) == 3 and filename.split('.')[1][
                0] in ['m', 'p']:  # Check if input is a superfit template
                return self.read_superfit_template()
            elif self.filename.split('-')[0] in list(catalogDict.keys()):  # Read input from catalog
                return catalogDict[self.filename[0:3]](self.filename)
            elif extension == self.filename or extension in ['flm', 'txt', 'dat']:
                return self.read_dat_file()
            elif extension == 'fits':
                return self.read_fits_file()
            elif extension == 'lnw':
                return self.snid_template_spectra_all()
            else:
                try:
                    return self.read_dat_file()
                except:
                    print("Invalid Input File")
                    return 0

    def two_col_input_spectrum(self, wave, flux, z):
        wave, flux = self.processingTools.deredshift_spectrum(wave, flux, z)

        mask = (wave >= self.w0) & (wave < self.w1)
        wave = wave[mask]
        flux = flux[mask]

        if not wave.any():
            raise Exception("The spectrum {0} with redshift {1} is out of the wavelength range {2}A to {3}A, "
                            "and cannot be classified. Please remove this object or change the input redshift of this"
                            " spectrum.".format(self.filename, z, int(self.w0), int(self.w1)))

        fluxNorm = (flux - min(flux)) / (max(flux) - min(flux))

        return wave, fluxNorm

    def snid_template_spectra_all(self):
        """lnw file"""
        with open(self.filename, 'r') as FileObj:
            for lineNum, line in enumerate(FileObj):
                # Read Header Info
                if lineNum == 0:
                    header = (line.strip('\n')).split(' ')
                    header = [x for x in header if x != '']
                    numAges, nwx, w0x, w1x, mostKnots, tname, dta, ttype, ittype, itstype = header
                    numAges, mostKnots = map(int, (numAges, mostKnots))
                    nk = np.zeros(numAges)
                    fmean = np.zeros(numAges)
                    xk = np.zeros((mostKnots, numAges))
                    yk = np.zeros((mostKnots, numAges))

                # Read Spline Info
                elif lineNum == 1:
                    splineInfo = (line.strip('\n')).split(' ')
                    splineInfo = [x for x in splineInfo if x != '']
                    for j in range(numAges):
                        nk[j], fmean[j] = (splineInfo[2 * j + 1], splineInfo[2 * j + 2])
                elif lineNum in range(2, mostKnots + 2):
                    splineInfo = (line.strip('\n')).split(' ')
                    splineInfo = [x for x in splineInfo if x != '']
                    for j in range(numAges):
                        xk[lineNum - 2, j], yk[lineNum - 2, j] = (splineInfo[2 * j + 1], splineInfo[2 * j + 2])

                elif lineNum == mostKnots + 2:
                    break

        splineInfo = (nk, fmean, xk, yk)

        # Read Normalized spectra
        if USE_PANDAS is True:
            arr = pd.read_csv(self.filename, skiprows=mostKnots + 2, header=None, delim_whitespace=True).values
        else:
            arr = np.loadtxt(self.filename, skiprows=mostKnots + 2)
        ages = arr[0]
        ages = np.delete(ages, 0)
        arr = np.delete(arr, 0, 0)

        wave = arr[:, 0]
        fluxes = np.zeros(shape=(numAges, len(arr)))  # initialise 2D array

        for i in range(0, len(arr[0]) - 1):
            fluxes[i] = arr[:, i + 1]

        if ttype == 'Ia-99aa':
            ttype = 'Ia-91T'

        return wave, fluxes, numAges, ages, ttype, splineInfo

    def snid_template_undo_processing(self, wave, flux, splineInfo, ageIdx):
        # Undo continuum removal -> then add galaxy -> then redshift
        nkAll, fmeanAll, xkAll, ykAll = splineInfo
        nk, fmean, xk, yk = int(nkAll[ageIdx]), fmeanAll[ageIdx], xkAll[:, ageIdx], ykAll[:, ageIdx]
        xk, yk = xk[:nk], yk[:nk]

        # NEED TO USE THIS TO ACTUALLY ADD THE SPLINE CONTINUUM BACK. NOT DOING ANYTHING AT THE MOMENT.

        return wave, flux


class PreProcessSpectrum(object):
    def __init__(self, w0, w1, nw):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.dwlog = np.log(w1 / w0) / nw
        self.processingTools = ProcessingTools()

    def log_wavelength(self, wave, flux):
        # Set up log wavelength array bins
        wlog = self.w0 * np.exp(np.arange(0, self.nw) * self.dwlog)

        fluxOut = self._vectorised_log_binning(wave, flux)
        # fluxOut = self._original_log_binning(wave, flux)

        minIndex, maxIndex = self.processingTools.min_max_index(fluxOut, outerVal=0)

        return wlog, fluxOut, minIndex, maxIndex

    def _vectorised_log_binning(self, wave, flux):
        """ Vectorised code version of the self._original_log_binning (for improved speed since this is the most called
        function in the script during training). This is complicated, but it has been tested to match the slower
        looping method """

        spec = np.array([wave, flux]).T
        mask = (wave >= self.w0) & (wave < self.w1)
        spec = spec[mask]
        wave, flux = spec.T
        try:
            fluxOut = np.zeros(int(self.nw))
            waveMiddle = wave[1:-1]
            waveTake1Index = wave[:-2]
            wavePlus1Index = wave[2:]
            s0List = 0.5 * (waveTake1Index + waveMiddle)
            s1List = 0.5 * (waveMiddle + wavePlus1Index)
            s0First = 0.5 * (3 * wave[0] - wave[1])
            s0Last = 0.5 * (wave[-2] + wave[-1])
            s1First = 0.5 * (wave[0] + wave[1])
            s1Last = 0.5 * (3 * wave[-1] - wave[-2])
            s0List = np.concatenate([[s0First], s0List, [s0Last]])
            s1List = np.concatenate([[s1First], s1List, [s1Last]])
            s0LogList = np.log(s0List / self.w0) / self.dwlog + 1
            s1LogList = np.log(s1List / self.w0) / self.dwlog + 1
            dnuList = s1List - s0List

            s0LogListInt = s0LogList.astype(int)
            s1LogListInt = s1LogList.astype(int)
            numOfJLoops = s1LogListInt - s0LogListInt
            jIndexes = np.flatnonzero(numOfJLoops)
            jIndexVals = s0LogListInt[jIndexes]
            prependZero = jIndexVals[0] if jIndexVals[0] < 0 else False
            if prependZero is not False:
                jIndexVals[0] = 0
                numOfJLoops[0] += prependZero
            numOfJLoops = (numOfJLoops[jIndexes])[jIndexVals < self.nw]
            fluxValList = ((flux * 1 / (s1LogList - s0LogList) * dnuList)[jIndexes])[jIndexVals < self.nw]
            fluxValList = np.repeat(fluxValList, numOfJLoops)
            minJ = min(jIndexVals)
            maxJ = (max(jIndexVals) + numOfJLoops[-1]) if (max(jIndexVals) + numOfJLoops[-1] < self.nw) else self.nw
            fluxOut[minJ:maxJ] = fluxValList[:(maxJ - minJ)]

            return fluxOut
        except Exception as e:
            print(e)
            print('wave', wave)
            print('flux', flux)
            print("########################################ERROR#######################################\n\n\n\n")
            return np.zeros(self.nw)

    def _original_log_binning(self, wave, flux):
        """ Rebin wavelengths: adapted from SNID apodize.f subroutine rebin() """
        fluxOut = np.zeros(int(self.nw))

        for i in range(0, len(wave)):
            if i == 0:
                s0 = 0.5 * (3 * wave[i] - wave[i + 1])
                s1 = 0.5 * (wave[i] + wave[i + 1])
            elif i == len(wave) - 1:
                s0 = 0.5 * (wave[i - 1] + wave[i])
                s1 = 0.5 * (3 * wave[i] - wave[i - 1])
            else:
                s0 = 0.5 * (wave[i - 1] + wave[i])
                s1 = 0.5 * (wave[i] + wave[i + 1])

            s0log = np.log(s0 / self.w0) / self.dwlog + 1
            s1log = np.log(s1 / self.w0) / self.dwlog + 1
            dnu = s1 - s0

            for j in range(int(s0log), int(s1log)):
                if j < 0 or j >= self.nw:
                    continue
                alen = 1  # min(s1log, j+1) - max(s0log, j)
                fluxval = flux[i] * alen / (s1log - s0log) * dnu
                fluxOut[j] = fluxOut[j] + fluxval

        return fluxOut

    def spline_fit(self, wave, flux, numSplinePoints, minindex, maxindex):
        continuum = np.zeros(int(self.nw)) + 1
        if (maxindex - minindex) > 5:
            spline = UnivariateSpline(wave[minindex:maxindex + 1], flux[minindex:maxindex + 1], k=3)
            splineWave = np.linspace(wave[minindex], wave[maxindex], num=numSplinePoints, endpoint=True)
            splinePoints = spline(splineWave)

            splineMore = UnivariateSpline(splineWave, splinePoints, k=3)
            splinePointsMore = splineMore(wave[minindex:maxindex + 1])

            continuum[minindex:maxindex + 1] = splinePointsMore
        else:
            print("WARNING: LESS THAN 6 POINTS IN SPECTRUM")

        return continuum

    def continuum_removal(self, wave, flux, numSplinePoints, minIndex, maxIndex):
        flux = flux + 1  # Important to keep this as +1
        contRemovedFlux = np.copy(flux)

        splineFit = self.spline_fit(wave, flux, numSplinePoints, minIndex, maxIndex)
        contRemovedFlux[minIndex:maxIndex + 1] = flux[minIndex:maxIndex + 1] / splineFit[minIndex:maxIndex + 1]
        contRemovedFluxNorm = normalise_spectrum(contRemovedFlux - 1)
        contRemovedFluxNorm = zero_non_overlap_part(contRemovedFluxNorm, minIndex, maxIndex)

        return contRemovedFluxNorm, splineFit - 1

    def mean_zero(self, flux, minindex, maxindex):
        """mean zero flux"""
        meanflux = np.mean(flux[minindex:maxindex])
        meanzeroflux = flux - meanflux
        meanzeroflux[0:minindex] = flux[0:minindex]
        meanzeroflux[maxindex + 1:] = flux[maxindex + 1:]

        return meanzeroflux

    def apodize(self, flux, minindex, maxindex, outerVal=0):
        """apodize with 5% cosine bell"""
        percent = 0.05
        fluxout = np.copy(flux) - outerVal

        nsquash = int(self.nw * percent)
        for i in range(0, nsquash):
            arg = np.pi * i / (nsquash - 1)
            factor = 0.5 * (1 - np.cos(arg))
            if (minindex + i < self.nw) and (maxindex - i >= 0):
                fluxout[minindex + i] = factor * fluxout[minindex + i]
                fluxout[maxindex - i] = factor * fluxout[maxindex - i]
            else:
                print("INVALID FLUX IN PREPROCESSING.PY APODIZE()")
                print("MININDEX=%d, i=%d" % (minindex, i))
                break

        if outerVal != 0:
            fluxout = fluxout + outerVal
            fluxout = zero_non_overlap_part(fluxout, minindex, maxindex, outerVal=outerVal)

        return fluxout


if __name__ == '__main__':
    # Plot comparison of superfit galaxies and Bsnip galaxies
    from astrodash.array_tools import normalise_spectrum
    import matplotlib.pyplot as plt

    preProcess = PreProcessSpectrum(3500, 10000, 1024)

    for gal in ['E', 'S0', 'Sa', 'Sb', 'Sc', 'SB1', 'SB2', 'SB3', 'SB4', 'SB5', 'SB6']:
        sfReadSpectrum = ReadSpectrumFile('../templates/superfit_templates/gal/%s' % gal, 3500, 10000, 1024)
        sfWave, sfFlux = sfReadSpectrum.file_extension()
        sfWave, sfFlux = sfReadSpectrum.two_col_input_spectrum(sfWave, sfFlux, z=0)
        # plt.plot(sfWave, sfFlux)
        sfWave, sfFlux, sfMinIndex, sfMaxIndex = preProcess.log_wavelength(sfWave, sfFlux)
        plt.plot(sfWave, normalise_spectrum(sfFlux))
        sfFlux, continuum = preProcess.continuum_removal(sfWave, sfFlux, 13, sfMinIndex, sfMaxIndex)
        # plt.plot(sfWave, continuum-1)

        snidReadSpectrum = ReadSpectrumFile('../templates/bsnip_v7_snid_templates/kc%s.lnw' % gal, 3500, 10000, 1024)
        snidSpectrum = snidReadSpectrum.file_extension()
        snidWave, snidFluxes, ncols, ages, ttype, splineInfo = snidSpectrum
        snidWave, snidFlux, snidMinIndex, snidMaxIndex = preProcess.log_wavelength(snidWave, snidFluxes[0])
        snidFlux = normalise_spectrum(snidFlux)
        fluxNorm = zero_non_overlap_part(snidFlux, snidMinIndex, snidMaxIndex, outerVal=0.5)

        plt.title(gal)
        plt.plot(sfWave, sfFlux, label='superfit')
        # plt.plot(snidWave, snidFlux, label='BSNIP')
        plt.plot(sfWave, normalise_spectrum(sfFlux * (continuum - 1)), label='superfit_continuumMultiply')
        plt.legend()
        plt.show()



import os
import glob
import astrodash
import pandas as pd
import numpy as np
import astropy.io.fits as afits


def read_all_atels(atelTextFile):
    """ Read the text file containing all OzDES ATel
    Returns the names, redshifts, type, phase and notes from the ATel
    atels Index(['Name', 'RA (J2000)', 'Dec (J2000)', 'Discovery Date (UT) ',
       'Discovery Mag (r)', 'Spectrum Date (UT)', 'Redshift', 'Type', 'Phase',
       'Notes'],
      dtype='object')
    """

    atels = pd.read_csv(atelTextFile, delimiter='|')
    atels['Name'] = atels['Name'].str.strip()
    pd.to_numeric(atels['Redshift'])
    atels['Filename'] = ''

    return atels


def main(spectraDir, atelTextFile, saveMatchesFilename):
    filenames, knownRedshifts, wikiClassifications = [], [], []

    # Store all the file paths to the objects in this directory
    allFilePaths = glob.glob('%s/*.dat' % spectraDir)

    # Get filenames and corresponding redshifts
    atels = read_all_atels(atelTextFile)
    for i, row in atels.iterrows():
        count = 0
        for filePath in allFilePaths:
            name = os.path.basename(filePath).replace('.dat', '').split('_')[0]
            if row.Name == name:
                filenames.append(filePath)
                knownRedshifts.append(row.Redshift)
                wikiClassifications.append("{} {}".format(row.Type, row.Phase))
                count += 1
                # break  # Uncomment the break to only classify the last dated spectrum for each object instead of classifying all dates.
        print(count)
        if count == 0:
            print(row.Name)

    # Classify and print the best matches
    classification = astrodash.Classify(filenames, knownRedshifts, classifyHost=False, rlapScores=True, smooth=16,
                                        knownZ=True, data_files='models_v06')
    bestFits, redshifts, bestTypes, rlapFlag, matchesFlag, redshiftErrs = classification.list_best_matches(n=5,
                                                                                                           saveFilename=saveMatchesFilename)

    # print("{0:17} | {1:5} | {2:8} | {3:10} | {4:6} | {5:10} | {6:10}".format("Name", "  z  ", "DASH_Fit", "  Age ", "Prob.", "Flag", "Wiki Fit"))
    # for i in range(len(filenames)):
    #     print("{0:17} | {1:5} | {2:8} | {3:10} | {4:6} | {5:10} | {6:10}".format('_'.join([filenames[i].split('/')[-1].split('_')[0], filenames[i].split('/')[-1].split('_')[3]]) , redshifts[i], bestTypes[i][0], bestTypes[i][1], bestTypes[i][2], matchesFlag[i].replace(' matches',''), wikiClassifications[i]))
    #
    print("{0:17} | {1:5} | {2:10} | {3:8} | {4:10} | {5:6} | {6:10} | {6:10} ".format("Name", "  z  ",
                                                                                       "ATel Classification",
                                                                                       "DASH_Class", "  Age ", "Prob.",
                                                                                       "Flag", "Best fit"))
    for i in range(len(filenames)):
        print("{0:17} | {1:5} | {2:10} | {3:8} | {4:10} | {5:6} | {6:10} | {6:10} ".format(
            '_'.join([filenames[i].split('/')[-1].split('_')[0], filenames[i].split('/')[-1].split('_')[3]]),
            redshifts[i], wikiClassifications[i], bestTypes[i][0], bestTypes[i][1], bestTypes[i][2],
            matchesFlag[i].replace(' matches', '')), bestFits[i][0])

    # print("{0:17} | {1:5} | {2:10} | {3:8} | {4:10} | {5:6} | {6:10} | {6:10} ".format("Name", "  z  ", "ATel Classification", "DASH_Class", "  Age ", "Prob.", "Flag", "Best fit"))
    # for i in np.arange(len(filenames)-1, -1, -1):
    #     name = filenames[i].split('/')[-1].split('_')[0]
    #     dashName, dashAge, dashProb = bestFits[i][0][1:4]
    #     print("\hline {0} & {1} & {2} & {3} ({4}) & {5} & {6} & \checkmark\\\\".format(
    #         name, round(redshifts[i], 3), wikiClassifications[i], dashName, dashAge, round(float(dashProb), 3), matchesFlag[i].replace(' matches','')))
    #

    # Plot one of the matches
    classification.plot_with_gui(indexToPlot=5)


if __name__ == '__main__':
    main(spectraDir='/Users/danmuth/PycharmProjects/astrodash/templates/OzDES_data/transients_all/',
         atelTextFile='/Users/danmuth/PycharmProjects/astrodash/templates/OzDES_data/all_atels.txt',
         saveMatchesFilename='DASH_matches_all_atels.txt')



import os
import pickle
import numpy as np
import scipy.interpolate
from astrodash.preprocessing import PreProcessSpectrum


def calc_params_for_log_redshifting(dataDirName):
    with open(os.path.join(dataDirName, 'training_params.pickle'), 'rb') as f1:
        pars = pickle.load(f1)
    w0, w1, nw = pars['w0'], pars['w1'], pars['nw']
    n = np.arange(0, int(nw))

    dwlog = np.log(w1 / w0) / nw
    # wlog = w0 * np.exp(n * dwlog)
    # waveInterp = scipy.interpolate.interp1d(n, wlog)  # Will interpolate any value within wavelength given an index

    return n, dwlog, w0, w1, nw


def redshift_binned_spectrum(flux, z, nIndexes, dwlog, w0, w1, nw, outerVal=0.5):
    # assert len(flux) == nw
    redshiftedIndexes = nIndexes + np.log(1 + z) / dwlog
    indexesInRange = redshiftedIndexes[redshiftedIndexes > 0]
    fluxInterp = scipy.interpolate.interp1d(indexesInRange, flux[redshiftedIndexes > 0], kind='linear')

    minWaveIndex = int(indexesInRange[0])

    fluxRedshifted = np.zeros(nw)
    fluxRedshifted[0:minWaveIndex] = outerVal * np.ones(minWaveIndex)
    fluxRedshifted[minWaveIndex:] = fluxInterp(indexesInRange)[:nw - minWaveIndex]

    # Apodize edges
    preprocess = PreProcessSpectrum(w0, w1, nw)
    minIndex, maxIndex = preprocess.processingTools.min_max_index(fluxRedshifted, outerVal=outerVal)
    apodizedFlux = preprocess.apodize(fluxRedshifted, minIndex, maxIndex, outerVal=outerVal)

    return apodizedFlux


def temp_list(tempFileList):
    f = open(tempFileList, 'rU')

    fileList = f.readlines()
    for i in range(0, len(fileList)):
        fileList[i] = fileList[i].strip('\n')

    f.close()

    return fileList


def div0(a, b):
    """ ignore / 0, div0( [-1, 0, 1], 0 ) -> [0, 0, 0] """
    with np.errstate(divide='ignore', invalid='ignore'):
        c = np.true_divide(a, b)
        c[~ np.isfinite(c)] = 0  # -inf inf NaN
    return c



from astrodash.training_parameters import create_training_params_file
from astrodash.create_training_set import create_training_set_files
from astrodash.create_template_set import create_template_set_file
from astrodash.deep_learning_multilayer import train_model
import zipfile
import os
import glob
import shutil
import time

scriptDirectory = os.path.dirname(os.path.abspath(__file__))


if __name__ == '__main__':
    modelName = 'testingZeroZ'
    trainWithHost = True
    classifyHost = False
    minZ = 0.
    maxZ = 0.
    redshiftDuringTraining = True
    trainFraction = 0.8
    numTrainBatches = 2000000
    # Do not change this unless we want to redshift before training.
    numOfRedshifts = 1

    if numOfRedshifts != 1:
        redshiftDuringTraining = False

    dataDirName = os.path.join(scriptDirectory, 'data_files_{0}/'.format(modelName))
    dataFilenames = []
    if not os.path.exists(dataDirName):
        os.makedirs(dataDirName)

    # MAKE INFO TEXT FILE ABOUT MODEL
    modelInfoFilename = dataDirName + "model_info.txt"
    with open(modelInfoFilename, "w") as f:
        f.write("Date Time: %s\n" % time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime()))
        f.write("Directory: %s\n" % dataDirName)
        f.write("Add Host: {}\n".format(trainWithHost))
        f.write("SN-Host fractions: [0.99, 0.98, 0.95, 0.93, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1]\n")
        f.write("Classify Host: {}\n".format(classifyHost))
        f.write("Redshift: Zero\n")
        f.write("Redshift Range: {} to {}\n".format(minZ, maxZ))
        f.write("Num of Redshifts: {}\n".format(numOfRedshifts))
        f.write("Redshift During Training: {}\n".format(redshiftDuringTraining))
        f.write("Fraction of Training Set Used: {}\n".format(trainFraction))
        f.write("Training Amount: 50 x {}\n".format(numTrainBatches))
        f.write("Changed wavelength range to 3500 to 10000A\n")
        f.write("Set outer region to 0.5\n")
        f.write("Using 2 convolutional layers in neural network\n")
        dataFilenames.append(modelInfoFilename)

    # CREATE PARAMETERS PICKLE FILE
    t1 = time.time()
    trainingParamsFilename = create_training_params_file(dataDirName)  # os.path.join(dataDirName, 'training_params.pickle')
    dataFilenames.append(trainingParamsFilename)
    t2 = time.time()
    print("time spent: {0:.2f}".format(t2 - t1))

    # CREATE TRAINING SET FILES
    trainingSetFilename = create_training_set_files(dataDirName, minZ=minZ, maxZ=maxZ, numOfRedshifts=numOfRedshifts, trainWithHost=trainWithHost, classifyHost=classifyHost, trainFraction=trainFraction)  # os.path.join(dataDirName, 'training_set.zip')
    dataFilenames.append(trainingSetFilename)
    t3 = time.time()
    print("time spent: {0:.2f}".format(t3 - t2))

    # TRAIN TENSORFLOW MODEL
    modelFilenames = train_model(dataDirName, overwrite=True, numTrainBatches=numTrainBatches, minZ=minZ, maxZ=maxZ, redshifting=redshiftDuringTraining)
    dataFilenames.extend(modelFilenames)
    t4 = time.time()
    print("time spent: {0:.2f}".format(t4 - t3))

    # SAVE ALL FILES TO ZIP FILE
    dataFilesZip = 'data_files_{0}.zip'.format(modelName)
    with zipfile.ZipFile(dataFilesZip, 'w') as myzip:
        for f in dataFilenames:
            myzip.write(f)

    modelZip = 'model_{0}.zip'.format(modelName)
    with zipfile.ZipFile(modelZip, 'w') as myzip:
        for f in dataFilenames[0:2] + dataFilenames[3:]:
            myzip.write(f)

    # # Delete temporary memory mapping files
    # for filename in glob.glob('shuffled*.dat'):
    #     os.remove(filename)
    # for filename in glob.glob('oversampled*.dat'):
    #     os.remove(filename)

    # Delete data_files folder since they are now in the zip files
    # for filename in filenames:
    #     os.remove(filename)



import numpy as np
from scipy.fftpack import fft
from astrodash.array_tools import mean_zero_spectra


def cross_correlation(inputFlux, tempFlux, nw, tempMinMaxIndex):
    tempFlux = mean_zero_spectra(tempFlux, tempMinMaxIndex[0], tempMinMaxIndex[1], nw)
    inputFourier = fft(inputFlux)
    tempFourier = fft(tempFlux)
    # kinput = 2 * np.pi / inputwave
    # ktemp = 2 * np.pi / tempwave

    product = inputFourier * np.conj(tempFourier)
    xCorr = fft(product)

    rmsInput = np.std(inputFourier)
    rmsTemp = np.std(tempFourier)

    xCorrNorm = (1. / (nw * rmsInput * rmsTemp)) * xCorr

    rmsXCorr = np.std(product)

    xCorrNormRearranged = np.concatenate((xCorrNorm[int(len(xCorrNorm) / 2):], xCorrNorm[0:int(len(xCorrNorm) / 2)]))

    # return xCorr, rmsInput, rmsTemp, xCorrNorm, rmsXCorr, xCorrNormRearranged
    return xCorrNormRearranged


def calc_redshift_from_crosscorr(crossCorr, nw, dwlog):
    # Find max peak while ignoring peaks that lead to negative redshifts
    deltaPeak = np.argmax(crossCorr[:int(nw / 2) + 1])

    # z = np.exp(deltaPeak * dwlog) - 1 #equation 13 of Blondin)
    zAxisIndex = np.concatenate((np.arange(-nw / 2, 0), np.arange(0, nw / 2)))
    if deltaPeak <= nw / 2:
        z = (np.exp(abs(zAxisIndex) * dwlog) - 1)[deltaPeak]
    else:
        z = -(np.exp(abs(zAxisIndex) * dwlog) - 1)[deltaPeak]

    return z, crossCorr


def get_redshift_axis(nw, dwlog):
    zAxisIndex = np.concatenate((np.arange(-nw / 2, 0), np.arange(0, nw / 2)))
    zAxis = np.zeros(nw)
    zAxis[0:int(nw / 2 - 1)] = -(np.exp(abs(zAxisIndex[0:int(nw / 2 - 1)]) * dwlog) - 1)
    zAxis[int(nw / 2):] = (np.exp(abs(zAxisIndex[int(nw / 2):]) * dwlog) - 1)
    zAxis = zAxis[::-1]

    return zAxis


def get_redshift(inputFlux, tempFlux, nw, dwlog, tempMinMaxIndex):
    crossCorr = cross_correlation(inputFlux, tempFlux, nw, tempMinMaxIndex)
    redshift, crossCorr = calc_redshift_from_crosscorr(crossCorr, nw, dwlog)

    return redshift, crossCorr


def get_median_redshift(inputFlux, tempFluxes, nw, dwlog, inputMinMaxIndex, tempMinMaxIndexes, tempNames, outerVal=0.5):
    inputFlux = mean_zero_spectra(inputFlux, inputMinMaxIndex[0], inputMinMaxIndex[1], nw)
    assert inputFlux[0] == 0.

    redshifts = []
    crossCorrs = {}

    for i, tempFlux in enumerate(tempFluxes):
        assert tempFlux[0] == outerVal

        redshift, crossCorr = get_redshift(inputFlux, tempFlux - outerVal, nw, dwlog, tempMinMaxIndexes[i])
        redshifts.append(redshift)
        crossCorrs[tempNames[i]] = crossCorr

    if redshifts != []:
        medianIndex = np.argsort(redshifts)[len(redshifts) // 2]
        medianRedshift = redshifts[medianIndex]
        medianName = tempNames[medianIndex]
        try:
            stdRedshift = np.std(redshifts)
        except Exception as e:
            print("Error calculating redshift error. See calculate_redshift.py.", e)
            stdRedshift = None
    else:
        return None, None, None, None

    if len(redshifts) >= 10:
        redshiftError = np.std(redshifts)
    else:
        pass  # redshiftError = 1/rlap * kz

    return medianRedshift, crossCorrs, medianName, stdRedshift



import numpy as np
from collections import OrderedDict
import json
from six.moves.urllib.request import urlopen
from astropy.time import Time


def read_osc_input(filename, template=False):
    """ self.filename in the form osc-name-ageIndex. E.g. osc-sn2002er-10
    TODO: Do error checking if input is invalid
    """
    osc, objName, ageIdx = filename.split('-')

    def read_json(url):
        response = urlopen(url)
        return json.loads(response.read(), object_pairs_hook=OrderedDict)

    # Redshift
    urlRedshift = "https://api.sne.space/" + objName + "/redshift/value"
    redshift = read_json(urlRedshift)
    redshift = float(redshift[next(iter(redshift))]['redshift'][0][0])

    if template is False:
        # Spectrum
        urlSpectrum = "https://api.sne.space/" + objName + "/spectra/time+data?item={0}".format(ageIdx)
        data = read_json(urlSpectrum)
        data = data[next(iter(data))]['spectra'][0][1]
        data = np.array(list(map(list, zip(*data)))).astype(np.float)
        if data.shape[0] == 3:
            wave, flux, fluxerr = data
        elif data.shape[0] == 2:
            wave, flux = data
        else:
            raise Exception("Error reading the given OSC input: {}. Check data at {}".format(filename, urlSpectrum))

        return wave, flux, redshift

    elif template is True:
        # Age Max
        urlAgeMax = "https://api.sne.space/" + objName + "/maxdate/value"
        ageMax = read_json(urlAgeMax)
        ageMax = ageMax[next(iter(ageMax))]['maxdate'][0][0]
        ageMax = Time(ageMax.replace('/', '-')).mjd

        # Type
        urlTType = "https://api.sne.space/" + objName + "/claimedtype/value"
        tType = read_json(urlTType)
        tType = tType[next(iter(tType))]['claimedtype']  # List of types... choose one [0][0]

        # Spectrum
        urlSpectrum = "https://api.sne.space/" + objName + "/spectra/time+data"
        data = read_json(urlSpectrum)
        data = data[next(iter(data))]['spectra']
        nCols = len(data)  # number of ages

        waves = []
        fluxes = []
        ages = []
        for datum in data:
            age, spectrum = datum
            age = age - ageMax
            ages.append(age)

            wave, flux = np.array(list(map(list, zip(*data)))).astype(np.float)
            wave = wave / (redshift + 1)  # De-redshift spectrum
            waves.append(wave)
            fluxes.append(flux)

        return waves, fluxes, nCols, ages, tType


# filename must start with catalog key followed by '-'. E.g. osc-OTHERINFO
catalogDict = {'osc': read_osc_input}

if __name__ == '__main__':
    read_osc_input('osc-sn2002er-10')
